@@ -1,16 +1,15 @@
-#!/usr/bin/python2.7 -u
+#!/usr/bin/python
 
-import sys
-import os
-import subprocess
-import time
-from datetime import datetime
-import shutil
-import tempfile
-import hashlib
-import re
-import logging
 import argparse
+import hashlib
+import logging
+import os
+import re
+import shutil
+import subprocess
+import sys
+import tempfile
+from datetime import datetime
 
 ################
 #### Kapacitor Variables
@@ -21,6 +20,8 @@ os.environ["GO15VENDOREXPERIMENT"] = "1"
 
 # PACKAGING VARIABLES
 PACKAGE_NAME = "kapacitor"
+USER = "kapacitor"
+GROUP = "kapacitor"
 INSTALL_ROOT_DIR = "/usr/bin"
 LOG_DIR = "/var/log/kapacitor"
 DATA_DIR = "/var/lib/kapacitor"
@@ -28,12 +29,12 @@ SCRIPT_DIR = "/usr/lib/kapacitor/scripts"
 
 INIT_SCRIPT = "scripts/init.sh"
 SYSTEMD_SCRIPT = "scripts/kapacitor.service"
+PREINST_SCRIPT = "scripts/pre-install.sh"
 POSTINST_SCRIPT = "scripts/post-install.sh"
 POSTUNINST_SCRIPT = "scripts/post-uninstall.sh"
 LOGROTATE_CONFIG = "etc/logrotate.d/kapacitor"
 BASH_COMPLETION_SH = "usr/share/bash-completion/completions/kapacitor"
 DEFAULT_CONFIG = "etc/kapacitor/kapacitor.conf"
-PREINST_SCRIPT = None
 
 # Default AWS S3 bucket for uploads
 DEFAULT_BUCKET = "dl.influxdata.com/kapacitor/artifacts"
@@ -46,13 +47,14 @@ VENDOR = "InfluxData"
 DESCRIPTION = "Time series data processing engine"
 
 # SCRIPT START
-go_vet_command = "go tool vet -composites=false"
+go_vet_command = "go vet ./..."
 prereqs = [ 'git', 'go' ]
 optional_prereqs = [ 'fpm', 'rpmbuild', 'gpg' ]
 
 fpm_common_args = "-f -s dir --log error \
  --vendor {} \
  --url {} \
+ --before-install {} \
  --after-install {} \
  --after-remove {} \
  --license {} \
@@ -60,9 +62,12 @@ fpm_common_args = "-f -s dir --log error \
  --config-files {} \
  --config-files {} \
  --directories {} \
+ --rpm-attr 755,{},{}:{} \
+ --rpm-attr 755,{},{}:{} \
  --description \"{}\"".format(
         VENDOR,
         PACKAGE_URL,
+        PREINST_SCRIPT,
         POSTINST_SCRIPT,
         POSTUNINST_SCRIPT,
         PACKAGE_LICENSE,
@@ -76,6 +81,8 @@ fpm_common_args = "-f -s dir --log error \
                          os.path.dirname(SCRIPT_DIR[1:]),
                          os.path.dirname(DEFAULT_CONFIG),
                     ]),
+        USER, GROUP, LOG_DIR,
+        USER, GROUP, DATA_DIR,
         DESCRIPTION)
 
 targets = {
@@ -153,8 +160,8 @@ def run_generate():
     logging.info("Running generate...")
     run("go install ./vendor/github.com/golang/protobuf/protoc-gen-go")
     run("go install ./vendor/github.com/benbjohnson/tmpl")
-    generate_cmd = ["go", "generate"]
-    generate_cmd.extend(go_list())
+    run("go install ./vendor/github.com/mailru/easyjson/easyjson")
+    generate_cmd = ["go", "generate", "./..."]
     p = subprocess.Popen(generate_cmd)
     code = p.wait()
     if code == 0:
@@ -164,22 +171,25 @@ def run_generate():
         logging.error("Generate failed.")
         return False
 
-def go_get(branch, update=False, no_uncommitted=False):
+def go_get():
     """
     Retrieve build dependencies or restore pinned dependencies.
     """
-    # All Kapacitor deps should be vendored so do not go get them
-    logging.info("Pretending to retrieve Go dependencies...")
-
-    # Check for uncommitted changes if no_uncommitted was given.
-    if no_uncommitted:
-        changes = run("git status --porcelain").strip()
-        if len(changes) > 0:
-            logging.error("There are un-committed changes in your local branch, --no-uncommited was given, cannot continue")
-            return False
-
+    # Nothing to do, all dependencies are vendored.
     return True
 
+def check_nochanges():
+    """
+    Check that there are no changes
+    """
+    changes = run("git status --porcelain").strip()
+    if len(changes) > 0:
+        logging.error("There are un-committed changes in your local branch, --no-uncommited was given, cannot continue")
+        logging.debug("Changes:\n{}".format(changes))
+        return False
+    return True
+
+
 def run_tests(race, parallel, timeout, no_vet):
     """Run the Go test suite on binary output.
     """
@@ -190,13 +200,13 @@ def run_tests(race, parallel, timeout, no_vet):
         logging.info("Using parallel: {}".format(parallel))
     if timeout is not None:
         logging.info("Using timeout: {}".format(timeout))
-    out = run("go fmt {}".format(' '.join(go_list())))
+    out = run("go fmt ./...")
     if len(out) > 0:
         logging.error("Code not formatted. Please use 'go fmt ./...' to fix formatting errors.")
         logging.error("{}".format(out))
         return False
     if not no_vet:
-        vet_cmd = go_vet_command + " {}".format(" ".join(go_list(relative=True)))
+        vet_cmd = go_vet_command
         out = run(vet_cmd)
         if len(out) > 0:
             logging.error("Go vet failed. Please run '{}' and fix any errors.".format(vet_cmd))
@@ -204,14 +214,14 @@ def run_tests(race, parallel, timeout, no_vet):
             return False
     else:
         logging.info("Skipping 'go vet' call...")
-    test_command = "go test -v"
+    test_command = "go test"
     if race:
         test_command += " -race"
     if parallel is not None:
         test_command += " -parallel {}".format(parallel)
     if timeout is not None:
         test_command += " -timeout {}".format(timeout)
-    test_command += " {}".format(' '.join(go_list()))
+    test_command += " ./..."
     logging.info("Running tests...")
     output = run(test_command, printOutput=logging.getLogger().getEffectiveLevel() == logging.DEBUG)
     return True
@@ -239,7 +249,7 @@ def package_python_udf(version, dist_dir):
     fname = "python-kapacitor_udf-{}.tar.gz".format(version)
     outfile = os.path.join(dist_dir, fname)
 
-    tar_cmd = ['tar', '-cz', '-C', './udf/agent/py', '--transform', 's/^./kapacitor_udf-{}/'.format(version), '-f']
+    tar_cmd = ['tar', '-cz', '-C', './udf/agent/py', '--owner=root', '--group=root', '--transform', 's/^./kapacitor_udf-{}/'.format(version), '-f']
     tar_cmd.append(outfile)
     exclude_list = ['*.pyc', '*.pyo', '__pycache__']
     for e in exclude_list:
@@ -477,32 +487,6 @@ def upload_packages(packages, bucket_name=None, overwrite=False):
             logging.warn("Not uploading file {}, as it already exists in the target bucket.".format(name))
     return True
 
-def go_list(vendor=False, relative=False):
-    """
-    Return a list of packages
-    If vendor is False vendor package are not included
-    If relative is True the package prefix defined by PACKAGE_URL is stripped
-    """
-    p = subprocess.Popen(["go", "list", "./..."], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    out, err = p.communicate()
-    packages = out.split('\n')
-    if packages[-1] == '':
-        packages = packages[:-1]
-    if not vendor:
-        non_vendor = []
-        for p in packages:
-            if '/vendor/' not in p:
-                non_vendor.append(p)
-        packages = non_vendor
-    if relative:
-        relative_pkgs = []
-        for p in packages:
-            r = p.replace(PACKAGE_URL, '.')
-            if r != '.':
-                relative_pkgs.append(r)
-        packages = relative_pkgs
-    return packages
-
 def build(version=None,
           platform=None,
           arch=None,
@@ -575,22 +559,22 @@ def build(version=None,
             build_command += "-tags {} ".format(','.join(tags))
         if "1.4" in get_go_version():
             if static:
-                build_command += "-ldflags=\"-s -X main.version {} -X main.branch {} -X main.commit {}\" ".format(version,
+                build_command += "-ldflags=\"-s -X main.version {} -X main.branch {} -X main.commit {} -X main.platform OSS\" ".format(version,
                                                                                                                   get_current_branch(),
                                                                                                                   get_current_commit())
             else:
-                build_command += "-ldflags=\"-X main.version {} -X main.branch {} -X main.commit {}\" ".format(version,
+                build_command += "-ldflags=\"-X main.version {} -X main.branch {} -X main.commit {} -X main.platform OSS\" ".format(version,
                                                                                                                get_current_branch(),
                                                                                                                get_current_commit())
 
         else:
             # Starting with Go 1.5, the linker flag arguments changed to 'name=value' from 'name value'
             if static:
-                build_command += "-ldflags=\"-s -X main.version={} -X main.branch={} -X main.commit={}\" ".format(version,
+                build_command += "-ldflags=\"-s -X main.version={} -X main.branch={} -X main.commit={} -X main.platform=OSS\" ".format(version,
                                                                                                                   get_current_branch(),
                                                                                                                   get_current_commit())
             else:
-                build_command += "-ldflags=\"-X main.version={} -X main.branch={} -X main.commit={}\" ".format(version,
+                build_command += "-ldflags=\"-X main.version={} -X main.branch={} -X main.commit={} -X main.platform=OSS\" ".format(version,
                                                                                                                get_current_branch(),
                                                                                                                get_current_commit())
         if static:
@@ -602,10 +586,10 @@ def build(version=None,
         logging.info("Time taken: {}s".format((end_time - start_time).total_seconds()))
     return True
 
-def generate_md5_from_file(path):
-    """Generate MD5 signature based on the contents of the file at path.
+def generate_sha256_from_file(path):
+    """Generate SHA256 signature based on the contents of the file at path.
     """
-    m = hashlib.md5()
+    m = hashlib.sha256()
     with open(path, 'rb') as f:
         for chunk in iter(lambda: f.read(4096), b""):
             m.update(chunk)
@@ -721,7 +705,7 @@ def package(build_output, pkg_name, version, nightly=False, iteration=1, static=
                                                             package_arch)
                         current_location = os.path.join(os.getcwd(), current_location)
                         if package_type == 'tar':
-                            tar_command = "cd {} && tar -cvzf {}.tar.gz ./*".format(package_build_root, name)
+                            tar_command = "cd {} && tar -cvzf {}.tar.gz --owner=root --group=root ./*".format(package_build_root, name)
                             run(tar_command, shell=True)
                             run("mv {}.tar.gz {}".format(os.path.join(package_build_root, name), current_location), shell=True)
                             outfile = os.path.join(current_location, name + ".tar.gz")
@@ -745,7 +729,7 @@ def package(build_output, pkg_name, version, nightly=False, iteration=1, static=
                             package_build_root,
                             current_location)
                         if package_type == "rpm":
-                            fpm_command += "--depends coreutils --rpm-posttrans {}".format(POSTINST_SCRIPT)
+                            fpm_command += "--depends coreutils --depends shadow-utils --rpm-posttrans {}".format(POSTINST_SCRIPT)
                         out = run(fpm_command, shell=True)
                         matches = re.search(':path=>"(.*)"', out)
                         outfile = None
@@ -759,13 +743,6 @@ def package(build_output, pkg_name, version, nightly=False, iteration=1, static=
                                 new_outfile = outfile.replace("{}-{}".format(package_version, package_iteration), "nightly")
                                 os.rename(outfile, new_outfile)
                                 outfile = new_outfile
-                            else:
-                                if package_type == 'rpm':
-                                    # rpm's convert any dashes to underscores
-                                    package_version = package_version.replace("-", "_")
-                                new_outfile = outfile.replace("{}-{}".format(package_version, package_iteration), package_version)
-                                os.rename(outfile, new_outfile)
-                                outfile = new_outfile
                             outfiles.append(os.path.join(os.getcwd(), outfile))
         logging.debug("Produced package files: {}".format(outfiles))
         return outfiles
@@ -819,12 +796,16 @@ def main(args):
         logging.info("Moving to git commit: {}".format(args.commit))
         run("git checkout {}".format(args.commit))
 
+    if not args.no_get:
+        if not go_get():
+            return 1
+
     if args.generate:
         if not run_generate():
             return 1
 
-    if not args.no_get:
-        if not go_get(args.branch, update=args.update, no_uncommitted=args.no_uncommitted):
+    if args.no_uncommitted:
+        if not check_nochanges():
             return 1
 
     if args.test:
@@ -898,8 +879,8 @@ def main(args):
                 return 1
         logging.info("Packages created:")
         for p in packages:
-            logging.info("{} (MD5={})".format(p.split('/')[-1:][0],
-                                              generate_md5_from_file(p)))
+            logging.info("{} (sha256={})".format(p.split('/')[-1:][0],
+                                              generate_sha256_from_file(p)))
 
 
     if orig_branch != get_current_branch():
