@@ -4,7 +4,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"log"
 	"net/http"
 	"net/http/httptest"
 	"os"
@@ -26,7 +25,8 @@ import (
 func TestBatch_InvalidQuery(t *testing.T) {
 
 	// Create a new execution env
-	tm := kapacitor.NewTaskMaster("invalidQuery", logService)
+	d := diagService.NewKapacitorHandler()
+	tm := kapacitor.NewTaskMaster("invalidQuery", newServerInfo(), d)
 	tm.HTTPDService = newHTTPDService()
 	tm.TaskStore = taskStore{}
 	tm.DeadmanService = deadman{}
@@ -61,6 +61,109 @@ func TestBatch_InvalidQuery(t *testing.T) {
 	}
 }
 
+//{"name":"packets","points":[
+// {"fields":{"value":"bad"},"time":"2015-10-18T00:00:00Z"},
+// {"fields":{"value":"good"},"time":"2015-10-18T00:00:02Z"},
+// {"fields":{"value":"good"},"time":"2015-10-18T00:00:04Z"},
+// {"fields":{"value2":"good"},"time":"2015-10-18T00:00:05Z"},
+// {"fields":{"value":"bad"},"time":"2015-10-18T00:00:06Z"},
+// {"fields":{"value":"good"},"time":"2015-10-18T00:00:08Z"}]}
+func TestBatch_ChangeDetect(t *testing.T) {
+
+	var script = `
+batch
+	|query('''
+		SELECT "value"
+		FROM "telegraf"."default".packets
+''')
+		.period(10s)
+		.every(10s)
+		.groupBy(time(2s))
+	|changeDetect('value')
+	|httpOut('TestBatch_ChangeDetect')
+`
+
+	er := models.Result{
+		Series: models.Rows{
+			{
+				Name:    "packets",
+				Tags:    nil,
+				Columns: []string{"time", "value"},
+				Values: [][]interface{}{
+					{
+						time.Date(1971, 1, 1, 0, 0, 0, 0, time.UTC),
+						"bad",
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 2, 0, time.UTC),
+						"good",
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 6, 0, time.UTC),
+						"bad",
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 8, 0, time.UTC),
+						"good",
+					},
+				},
+			},
+		},
+	}
+
+	testBatcherWithOutput(t, "TestBatch_ChangeDetect", script, 21*time.Second, er, false)
+}
+
+func TestBatch_ChangeDetect_Many(t *testing.T) {
+
+	var script = `
+batch
+	|query('''
+		SELECT "value"
+		FROM "telegraf"."default".packets
+''')
+		.period(10s)
+		.every(10s)
+		.groupBy(time(2s))
+	|changeDetect('a','b')
+	|httpOut('TestBatch_ChangeDetect_Many')
+`
+
+	er := models.Result{
+		Series: models.Rows{
+			{
+				Name:    "packets",
+				Tags:    nil,
+				Columns: []string{"time", "a", "b"},
+				Values: [][]interface{}{
+					[]interface{}{
+						time.Date(1971, 1, 1, 0, 0, 0, 0, time.UTC),
+						"bad",
+						0.0,
+					},
+					[]interface{}{
+						time.Date(1971, 1, 1, 0, 0, 2, 0, time.UTC),
+						"good",
+						0.0,
+					},
+					[]interface{}{
+						time.Date(1971, 1, 1, 0, 0, 6, 0, time.UTC),
+						"bad",
+						1.0,
+					},
+					[]interface{}{
+						time.Date(1971, 1, 1, 0, 0, 8, 0, time.UTC),
+						"bad",
+						0.0,
+					},
+				},
+			},
+		},
+	}
+
+	testBatcherWithOutput(t, "TestBatch_ChangeDetect_Many", script, 21*time.Second, er, false)
+}
+
 func TestBatch_Derivative(t *testing.T) {
 
 	var script = `
@@ -350,7 +453,6 @@ batch
 		.period(10s)
 		.every(10s)
 	|difference('value')
-	|log()
 	|httpOut('TestBatch_Difference')
 `
 
@@ -1450,10 +1552,12 @@ func TestBatch_AlertStateChangesOnly(t *testing.T) {
 		atomic.AddInt32(&requestCount, 1)
 		if rc := atomic.LoadInt32(&requestCount); rc == 1 {
 			expAd := alert.Data{
-				ID:      "cpu_usage_idle:cpu=cpu-total",
-				Message: "cpu_usage_idle:cpu=cpu-total is CRITICAL",
-				Time:    time.Date(1971, 1, 1, 0, 0, 0, 0, time.UTC),
-				Level:   alert.Critical,
+				ID:            "cpu_usage_idle:cpu=cpu-total",
+				Message:       "cpu_usage_idle:cpu=cpu-total is CRITICAL",
+				Time:          time.Date(1971, 1, 1, 0, 0, 0, 0, time.UTC),
+				Level:         alert.Critical,
+				PreviousLevel: alert.OK,
+				Recoverable:   true,
 			}
 			ad.Data = models.Result{}
 			if eq, msg := compareAlertData(expAd, ad); !eq {
@@ -1461,11 +1565,13 @@ func TestBatch_AlertStateChangesOnly(t *testing.T) {
 			}
 		} else {
 			expAd := alert.Data{
-				ID:       "cpu_usage_idle:cpu=cpu-total",
-				Message:  "cpu_usage_idle:cpu=cpu-total is OK",
-				Time:     time.Date(1971, 1, 1, 0, 0, 38, 0, time.UTC),
-				Duration: 38 * time.Second,
-				Level:    alert.OK,
+				ID:            "cpu_usage_idle:cpu=cpu-total",
+				Message:       "cpu_usage_idle:cpu=cpu-total is OK",
+				Time:          time.Date(1971, 1, 1, 0, 0, 38, 0, time.UTC),
+				Duration:      38 * time.Second,
+				Level:         alert.OK,
+				PreviousLevel: alert.Critical,
+				Recoverable:   true,
 			}
 			ad.Data = models.Result{}
 			if eq, msg := compareAlertData(expAd, ad); !eq {
@@ -1517,21 +1623,36 @@ func TestBatch_AlertStateChangesOnlyExpired(t *testing.T) {
 		var expAd alert.Data
 		atomic.AddInt32(&requestCount, 1)
 		rc := atomic.LoadInt32(&requestCount)
-		if rc < 3 {
+		switch rc {
+		case 1:
 			expAd = alert.Data{
-				ID:       "cpu_usage_idle:cpu=cpu-total",
-				Message:  "cpu_usage_idle:cpu=cpu-total is CRITICAL",
-				Time:     time.Date(1971, 1, 1, 0, 0, int(rc-1)*20, 0, time.UTC),
-				Duration: time.Duration(rc-1) * 20 * time.Second,
-				Level:    alert.Critical,
+				ID:            "cpu_usage_idle:cpu=cpu-total",
+				Message:       "cpu_usage_idle:cpu=cpu-total is CRITICAL",
+				Time:          time.Date(1971, 1, 1, 0, 0, int(rc-1)*20, 0, time.UTC),
+				Duration:      time.Duration(rc-1) * 20 * time.Second,
+				Level:         alert.Critical,
+				PreviousLevel: alert.OK,
+				Recoverable:   true,
 			}
-		} else {
+		case 2:
 			expAd = alert.Data{
-				ID:       "cpu_usage_idle:cpu=cpu-total",
-				Message:  "cpu_usage_idle:cpu=cpu-total is OK",
-				Time:     time.Date(1971, 1, 1, 0, 0, 38, 0, time.UTC),
-				Duration: 38 * time.Second,
-				Level:    alert.OK,
+				ID:            "cpu_usage_idle:cpu=cpu-total",
+				Message:       "cpu_usage_idle:cpu=cpu-total is CRITICAL",
+				Time:          time.Date(1971, 1, 1, 0, 0, int(rc-1)*20, 0, time.UTC),
+				Duration:      time.Duration(rc-1) * 20 * time.Second,
+				Level:         alert.Critical,
+				PreviousLevel: alert.Critical,
+				Recoverable:   true,
+			}
+		case 3:
+			expAd = alert.Data{
+				ID:            "cpu_usage_idle:cpu=cpu-total",
+				Message:       "cpu_usage_idle:cpu=cpu-total is OK",
+				Time:          time.Date(1971, 1, 1, 0, 0, 38, 0, time.UTC),
+				Duration:      38 * time.Second,
+				Level:         alert.OK,
+				PreviousLevel: alert.Critical,
+				Recoverable:   true,
 			}
 		}
 		if eq, msg := compareAlertData(expAd, ad); !eq {
@@ -2967,6 +3088,177 @@ batch
 	testBatcherWithOutput(t, "TestBatch_HttpPost", script, 30*time.Second, er, false)
 }
 
+func TestBatch_HttpPost_Timeout(t *testing.T) {
+	ts := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		result := models.Result{}
+		dec := json.NewDecoder(r.Body)
+		err := dec.Decode(&result)
+		if err != nil {
+			t.Fatal(err)
+		}
+		time.Sleep(100 * time.Millisecond)
+	}))
+	defer ts.Close()
+
+	var script = `
+batch
+	|query('''
+		SELECT mean("value")
+		FROM "telegraf"."default".cpu_usage_idle
+		WHERE "host" = 'serverA' AND "cpu" != 'cpu-total'
+''')
+		.period(10s)
+		.every(10s)
+		.groupBy(time(2s), 'cpu')
+	|httpPost('` + ts.URL + `').timeout(1ms)
+	|httpOut('TestBatch_HttpPost_Timeout')
+`
+
+	er := models.Result{
+		Series: models.Rows{
+			{
+				Name:    "cpu_usage_idle",
+				Tags:    map[string]string{"cpu": "cpu-total"},
+				Columns: []string{"time", "mean"},
+				Values: [][]interface{}{
+					{
+						time.Date(1971, 1, 1, 0, 0, 20, 0, time.UTC),
+						91.06416290101595,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 22, 0, time.UTC),
+						85.9694442394385,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 24, 0, time.UTC),
+						90.62985736134186,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 26, 0, time.UTC),
+						86.45443196005628,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 28, 0, time.UTC),
+						88.97243107764031,
+					},
+				},
+			},
+			{
+				Name:    "cpu_usage_idle",
+				Tags:    map[string]string{"cpu": "cpu0"},
+				Columns: []string{"time", "mean"},
+				Values: [][]interface{}{
+					{
+						time.Date(1971, 1, 1, 0, 0, 20, 0, time.UTC),
+						85.08910891088406,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 22, 0, time.UTC),
+						78.00000000002001,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 24, 0, time.UTC),
+						84.23607066586464,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 26, 0, time.UTC),
+						80.85858585861834,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 28, 0, time.UTC),
+						80.61224489791657,
+					},
+				},
+			},
+			{
+				Name:    "cpu_usage_idle",
+				Tags:    map[string]string{"cpu": "cpu1"},
+				Columns: []string{"time", "mean"},
+				Values: [][]interface{}{
+					{
+						time.Date(1971, 1, 1, 0, 0, 20, 0, time.UTC),
+						96.49999999996908,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 22, 0, time.UTC),
+						93.46464646468584,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 24, 0, time.UTC),
+						95.00950095007724,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 26, 0, time.UTC),
+						92.99999999998636,
+					},
+					{
+						time.Date(1971, 1, 1, 0, 0, 28, 0, time.UTC),
+						90.99999999998545,
+					},
+				},
+			},
+		},
+	}
+
+	c := make(chan bool, 1)
+	go func() {
+		testBatcherWithOutput(t, "TestBatch_HttpPost_Timeout", script, 30*time.Second, er, false)
+		c <- true
+	}()
+	select {
+	case <-c:
+	case <-time.After(time.Second):
+		t.Fatal("Test timeout reached, httpPost().timeout() may not be functioning")
+	}
+}
+
+func TestBatch_AlertPost_Timeout(t *testing.T) {
+	ts := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		ad := alert.Data{}
+		dec := json.NewDecoder(r.Body)
+		err := dec.Decode(&ad)
+		if err != nil {
+			t.Fatal(err)
+		}
+		time.Sleep(time.Second)
+	}))
+	defer ts.Close()
+	var script = `
+batch
+	|query('''
+		SELECT mean("value")
+		FROM "telegraf"."default".cpu_usage_idle
+		WHERE "host" = 'serverA' AND "cpu" != 'cpu-total'
+''')
+		.period(10s)
+		.every(10s)
+		.groupBy(time(2s), 'cpu')
+	|alert()
+		.crit(lambda:"mean" > 90)
+		.stateChangesOnly()
+		.levelField('level')
+		.details('')
+		.post('` + ts.URL + `').timeout(1ms)
+`
+
+	c := make(chan bool, 1)
+	go func() {
+		clock, et, replayErr, tm := testBatcher(t, "TestBatch_AlertPostTimeout", script)
+		defer tm.Close()
+
+		err := fastForwardTask(clock, et, replayErr, tm, 40*time.Second)
+		if err != nil {
+			t.Error(err)
+		}
+		c <- true
+	}()
+	select {
+	case <-c:
+	case <-time.After(time.Second):
+		t.Fatal("Test timeout reached, alert().post().timeout() may not be functioning")
+	}
+}
+
 // Helper test function for batcher
 func testBatcher(t *testing.T, name, script string) (clock.Setter, *kapacitor.ExecutingTask, <-chan error, *kapacitor.TaskMaster) {
 	if testing.Verbose() {
@@ -2976,13 +3268,14 @@ func testBatcher(t *testing.T, name, script string) (clock.Setter, *kapacitor.Ex
 	}
 
 	// Create a new execution env
-	tm := kapacitor.NewTaskMaster("testBatcher", logService)
+	d := diagService.NewKapacitorHandler()
+	tm := kapacitor.NewTaskMaster("testBatcher", newServerInfo(), d)
 	httpdService := newHTTPDService()
 	tm.HTTPDService = httpdService
 	tm.TaskStore = taskStore{}
 	tm.DeadmanService = deadman{}
-	tm.HTTPPostService = httppost.NewService(nil, logService.NewLogger("[httppost] ", log.LstdFlags))
-	as := alertservice.NewService(logService.NewLogger("[alert] ", log.LstdFlags))
+	tm.HTTPPostService, _ = httppost.NewService(nil, diagService.NewHTTPPostHandler())
+	as := alertservice.NewService(diagService.NewAlertServiceHandler())
 	as.StorageService = storagetest.New()
 	as.HTTPDService = httpdService
 	if err := as.Open(); err != nil {
@@ -3002,7 +3295,7 @@ func testBatcher(t *testing.T, name, script string) (clock.Setter, *kapacitor.Ex
 	var data io.ReadCloser
 	for i := 0; err == nil; {
 		f := fmt.Sprintf("%s.%d.brpl", name, i)
-		data, err = os.Open(path.Join("data", f))
+		data, err = os.Open(path.Join("testdata", f))
 		if err == nil {
 			allData = append(allData, data)
 			i++
