@@ -1,21 +1,28 @@
 package alert
 
 import (
+	"encoding"
 	"encoding/json"
 	"fmt"
-	"log"
 	"path"
+	"reflect"
 	"regexp"
 	"sync"
 
 	"github.com/influxdata/kapacitor/alert"
 	"github.com/influxdata/kapacitor/command"
+	"github.com/influxdata/kapacitor/keyvalue"
+	"github.com/influxdata/kapacitor/models"
 	"github.com/influxdata/kapacitor/services/alerta"
 	"github.com/influxdata/kapacitor/services/hipchat"
 	"github.com/influxdata/kapacitor/services/httpd"
 	"github.com/influxdata/kapacitor/services/httppost"
+	"github.com/influxdata/kapacitor/services/kafka"
+	"github.com/influxdata/kapacitor/services/mqtt"
 	"github.com/influxdata/kapacitor/services/opsgenie"
+	"github.com/influxdata/kapacitor/services/opsgenie2"
 	"github.com/influxdata/kapacitor/services/pagerduty"
+	"github.com/influxdata/kapacitor/services/pagerduty2"
 	"github.com/influxdata/kapacitor/services/pushover"
 	"github.com/influxdata/kapacitor/services/sensu"
 	"github.com/influxdata/kapacitor/services/slack"
@@ -28,11 +35,24 @@ import (
 	"github.com/pkg/errors"
 )
 
+type Diagnostic interface {
+	WithHandlerContext(ctx ...keyvalue.T) HandlerDiagnostic
+
+	MigratingHandlerSpecs()
+	FoundHandlerRows(length int)
+	FoundNewHandler(key string)
+	CreatingNewHandlers(length int)
+	MigratingOldHandlerSpec(id string)
+
+	Error(msg string, err error, ctx ...keyvalue.T)
+}
+
 type Service struct {
 	mu sync.RWMutex
 
-	specsDAO  HandlerSpecDAO
-	topicsDAO TopicStateDAO
+	specsDAO      HandlerSpecDAO
+	topicsDAO     TopicStateDAO
+	PersistTopics bool
 
 	APIServer *apiServer
 
@@ -40,11 +60,13 @@ type Service struct {
 
 	closedTopics map[string]bool
 
+	inhibitorLookup *alert.InhibitorLookup
+
 	topics         *alert.Topics
 	EventCollector EventCollector
 
 	HTTPDService interface {
-		AddPreviewRoutes([]httpd.Route) error
+		AddRoutes([]httpd.Route) error
 		DelRoutes([]httpd.Route)
 	}
 
@@ -56,62 +78,75 @@ type Service struct {
 
 	Commander command.Commander
 
-	logger *log.Logger
+	diag Diagnostic
 
 	AlertaService interface {
 		DefaultHandlerConfig() alerta.HandlerConfig
-		Handler(alerta.HandlerConfig, *log.Logger) (alert.Handler, error)
+		Handler(alerta.HandlerConfig, ...keyvalue.T) (alert.Handler, error)
 	}
 	HipChatService interface {
-		Handler(hipchat.HandlerConfig, *log.Logger) alert.Handler
+		Handler(hipchat.HandlerConfig, ...keyvalue.T) alert.Handler
+	}
+	KafkaService interface {
+		Handler(kafka.HandlerConfig, ...keyvalue.T) (alert.Handler, error)
+	}
+	MQTTService interface {
+		Handler(mqtt.HandlerConfig, ...keyvalue.T) (alert.Handler, error)
 	}
 	OpsGenieService interface {
-		Handler(opsgenie.HandlerConfig, *log.Logger) alert.Handler
+		Handler(opsgenie.HandlerConfig, ...keyvalue.T) alert.Handler
+	}
+	OpsGenie2Service interface {
+		Handler(opsgenie2.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 	PagerDutyService interface {
-		Handler(pagerduty.HandlerConfig, *log.Logger) alert.Handler
+		Handler(pagerduty.HandlerConfig, ...keyvalue.T) alert.Handler
+	}
+	PagerDuty2Service interface {
+		Handler(pagerduty2.HandlerConfig, ...keyvalue.T) (alert.Handler, error)
 	}
 	PushoverService interface {
-		Handler(pushover.HandlerConfig, *log.Logger) alert.Handler
+		Handler(pushover.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 	HTTPPostService interface {
-		Handler(httppost.HandlerConfig, *log.Logger) alert.Handler
+		Handler(httppost.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 	SensuService interface {
-		Handler(sensu.HandlerConfig, *log.Logger) (alert.Handler, error)
+		Handler(sensu.HandlerConfig, ...keyvalue.T) (alert.Handler, error)
 	}
 	SlackService interface {
-		Handler(slack.HandlerConfig, *log.Logger) alert.Handler
+		Handler(slack.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 	SMTPService interface {
-		Handler(smtp.HandlerConfig, *log.Logger) alert.Handler
+		Handler(smtp.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 	SNMPTrapService interface {
-		Handler(snmptrap.HandlerConfig, *log.Logger) (alert.Handler, error)
+		Handler(snmptrap.HandlerConfig, ...keyvalue.T) (alert.Handler, error)
 	}
 	TalkService interface {
-		Handler(*log.Logger) alert.Handler
+		Handler(...keyvalue.T) alert.Handler
 	}
 	TelegramService interface {
-		Handler(telegram.HandlerConfig, *log.Logger) alert.Handler
+		Handler(telegram.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 	VictorOpsService interface {
-		Handler(victorops.HandlerConfig, *log.Logger) alert.Handler
+		Handler(victorops.HandlerConfig, ...keyvalue.T) alert.Handler
 	}
 }
 
-func NewService(l *log.Logger) *Service {
+func NewService(d Diagnostic) *Service {
 	s := &Service{
-		handlers:     make(map[string]map[string]handler),
-		closedTopics: make(map[string]bool),
-		topics:       alert.NewTopics(l),
-		logger:       l,
+		handlers:        make(map[string]map[string]handler),
+		closedTopics:    make(map[string]bool),
+		topics:          alert.NewTopics(),
+		diag:            d,
+		inhibitorLookup: alert.NewInhibitorLookup(),
 	}
 	s.APIServer = &apiServer{
 		Registrar: s,
 		Topics:    s,
 		Persister: s,
-		logger:    l,
+		diag:      d,
 	}
 	s.EventCollector = s
 	return s
@@ -189,7 +224,7 @@ func (s *Service) migrateHandlerSpecs(store storage.Interface) error {
 		// Already migrated
 		return nil
 	}
-	s.logger.Println("D! migrating old v1.2 handler specs")
+	s.diag.MigratingHandlerSpecs()
 
 	// v1.2 HandlerActionSpec
 	type oldHandlerActionSpec struct {
@@ -221,12 +256,12 @@ func (s *Service) migrateHandlerSpecs(store storage.Interface) error {
 		if err != nil {
 			return err
 		}
-		s.logger.Printf("D! found %d handler rows", len(kvs))
+		s.diag.FoundHandlerRows(len(kvs))
 
 		var oldKeys []string
 		for _, kv := range kvs {
 			if !oldKeyPattern.MatchString(kv.Key) {
-				s.logger.Println("D! found new handler skipping:", kv.Key)
+				s.diag.FoundNewHandler(kv.Key)
 				continue
 			}
 			oldKeys = append(oldKeys, kv.Key)
@@ -241,7 +276,7 @@ func (s *Service) migrateHandlerSpecs(store storage.Interface) error {
 				return errors.Wrapf(err, "failed to read old handler spec data for %s", kv.Key)
 			}
 
-			s.logger.Println("D! migrating old handler spec", old.ID)
+			s.diag.MigratingOldHandlerSpec(old.ID)
 
 			// Create new handlers from the old
 			hasStateChangesOnly := false
@@ -283,7 +318,7 @@ func (s *Service) migrateHandlerSpecs(store storage.Interface) error {
 			}
 		}
 
-		s.logger.Printf("D! creating %d new handlers in place of old handlers", len(newHandlers))
+		s.diag.CreatingNewHandlers(len(newHandlers))
 
 		// Create new handlers
 		for _, handler := range newHandlers {
@@ -311,7 +346,7 @@ func (s *Service) loadSavedHandlerSpecs() error {
 
 		for _, spec := range specs {
 			if err := s.loadHandlerSpec(spec); err != nil {
-				s.logger.Println("E! failed to load handler on startup", err)
+				s.diag.Error("failed to load handler on startup", err)
 			}
 		}
 
@@ -412,6 +447,10 @@ func (s *Service) Collect(event alert.Event) error {
 }
 
 func (s *Service) persistTopicState(topic string) error {
+	if !s.PersistTopics {
+		return nil
+	}
+
 	t, ok := s.topics.Topic(topic)
 	if !ok {
 		// Topic was deleted since event was collected, nothing to do.
@@ -667,7 +706,7 @@ func decodeOptions(options map[string]interface{}, c interface{}) error {
 	dec, err := mapstructure.NewDecoder(&mapstructure.DecoderConfig{
 		ErrorUnused: true,
 		Result:      c,
-		DecodeHook:  mapstructure.StringToTimeDurationHookFunc(),
+		DecodeHook:  decodeStringToTextUnmarshaler,
 	})
 	if err != nil {
 		return errors.Wrap(err, "failed to initialize mapstructure decoder")
@@ -678,9 +717,43 @@ func decodeOptions(options map[string]interface{}, c interface{}) error {
 	return nil
 }
 
+var textUnmarshalerType = reflect.TypeOf((*encoding.TextUnmarshaler)(nil)).Elem()
+
+// decodeStringToTextUnmarshaler will decode a string value into any type
+// that implements the encoding.TextUnmarshaler interface.
+func decodeStringToTextUnmarshaler(f, t reflect.Type, data interface{}) (interface{}, error) {
+	if f.Kind() != reflect.String {
+		return data, nil
+	}
+	isPtr := true
+	if t.Kind() != reflect.Ptr {
+		isPtr = false
+		t = reflect.PtrTo(t)
+	}
+	if t.Implements(textUnmarshalerType) {
+		value := reflect.New(t.Elem())
+		tum := value.Interface().(encoding.TextUnmarshaler)
+		str := data.(string)
+		err := tum.UnmarshalText([]byte(str))
+		if err != nil {
+			return nil, err
+		}
+
+		if isPtr {
+			return value.Interface(), nil
+		}
+		return reflect.Indirect(value).Interface(), nil
+	}
+	return data, nil
+}
+
 func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 	var h alert.Handler
 	var err error
+	ctx := []keyvalue.T{
+		keyvalue.KV("handler", spec.ID),
+		keyvalue.KV("topic", spec.Topic),
+	}
 	switch spec.Kind {
 	case "aggregate":
 		c := newDefaultAggregateHandlerConfig(s.EventCollector)
@@ -688,7 +761,8 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h, err = NewAggregateHandler(c, s.logger)
+		handlerDiag := s.diag.WithHandlerContext(ctx...)
+		h, err = NewAggregateHandler(c, handlerDiag)
 		if err != nil {
 			return handler{}, err
 		}
@@ -698,7 +772,7 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h, err = s.AlertaService.Handler(c, s.logger)
+		h, err = s.AlertaService.Handler(c, ctx...)
 		if err != nil {
 			return handler{}, err
 		}
@@ -711,7 +785,8 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = NewExecHandler(c, s.logger)
+		handlerDiag := s.diag.WithHandlerContext(ctx...)
+		h = NewExecHandler(c, handlerDiag)
 		h = newExternalHandler(h)
 	case "hipchat":
 		c := hipchat.HandlerConfig{}
@@ -719,7 +794,18 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.HipChatService.Handler(c, s.logger)
+		h = s.HipChatService.Handler(c, ctx...)
+		h = newExternalHandler(h)
+	case "kafka":
+		c := kafka.HandlerConfig{}
+		err = decodeOptions(spec.Options, &c)
+		if err != nil {
+			return handler{}, err
+		}
+		h, err = s.KafkaService.Handler(c, ctx...)
+		if err != nil {
+			return handler{}, err
+		}
 		h = newExternalHandler(h)
 	case "log":
 		c := DefaultLogHandlerConfig()
@@ -727,7 +813,19 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h, err = NewLogHandler(c, s.logger)
+		handlerDiag := s.diag.WithHandlerContext(ctx...)
+		h, err = NewLogHandler(c, handlerDiag)
+		if err != nil {
+			return handler{}, err
+		}
+		h = newExternalHandler(h)
+	case "mqtt":
+		c := mqtt.HandlerConfig{}
+		err = decodeOptions(spec.Options, &c)
+		if err != nil {
+			return handler{}, err
+		}
+		h, err = s.MQTTService.Handler(c, ctx...)
 		if err != nil {
 			return handler{}, err
 		}
@@ -738,7 +836,15 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.OpsGenieService.Handler(c, s.logger)
+		h = s.OpsGenieService.Handler(c, ctx...)
+		h = newExternalHandler(h)
+	case "opsgenie2":
+		c := opsgenie2.HandlerConfig{}
+		err = decodeOptions(spec.Options, &c)
+		if err != nil {
+			return handler{}, err
+		}
+		h = s.OpsGenie2Service.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	case "pagerduty":
 		c := pagerduty.HandlerConfig{}
@@ -746,7 +852,18 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.PagerDutyService.Handler(c, s.logger)
+		h = s.PagerDutyService.Handler(c, ctx...)
+		h = newExternalHandler(h)
+	case "pagerduty2":
+		c := pagerduty2.HandlerConfig{}
+		err = decodeOptions(spec.Options, &c)
+		if err != nil {
+			return handler{}, err
+		}
+		h, err = s.PagerDuty2Service.Handler(c, ctx...)
+		if err != nil {
+			return handler{}, err
+		}
 		h = newExternalHandler(h)
 	case "pushover":
 		c := pushover.HandlerConfig{}
@@ -754,7 +871,7 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.PushoverService.Handler(c, s.logger)
+		h = s.PushoverService.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	case "post":
 		c := httppost.HandlerConfig{}
@@ -762,7 +879,7 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.HTTPPostService.Handler(c, s.logger)
+		h = s.HTTPPostService.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	case "publish":
 		c := PublishHandlerConfig{
@@ -772,14 +889,15 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = NewPublishHandler(c, s.logger)
+		handlerDiag := s.diag.WithHandlerContext(ctx...)
+		h = NewPublishHandler(c, handlerDiag)
 	case "sensu":
 		c := sensu.HandlerConfig{}
 		err = decodeOptions(spec.Options, &c)
 		if err != nil {
 			return handler{}, err
 		}
-		h, err = s.SensuService.Handler(c, s.logger)
+		h, err = s.SensuService.Handler(c, ctx...)
 		if err != nil {
 			return handler{}, err
 		}
@@ -790,7 +908,7 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.SlackService.Handler(c, s.logger)
+		h = s.SlackService.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	case "smtp":
 		c := smtp.HandlerConfig{}
@@ -798,7 +916,7 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.SMTPService.Handler(c, s.logger)
+		h = s.SMTPService.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	case "snmptrap":
 		c := snmptrap.HandlerConfig{}
@@ -806,13 +924,13 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h, err = s.SNMPTrapService.Handler(c, s.logger)
+		h, err = s.SNMPTrapService.Handler(c, ctx...)
 		if err != nil {
 			return handler{}, err
 		}
 		h = newExternalHandler(h)
 	case "talk":
-		h = s.TalkService.Handler(s.logger)
+		h = s.TalkService.Handler(ctx...)
 		h = newExternalHandler(h)
 	case "tcp":
 		c := TCPHandlerConfig{}
@@ -820,7 +938,8 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = NewTCPHandler(c, s.logger)
+		handlerDiag := s.diag.WithHandlerContext(ctx...)
+		h = NewTCPHandler(c, handlerDiag)
 		h = newExternalHandler(h)
 	case "telegram":
 		c := telegram.HandlerConfig{}
@@ -828,7 +947,7 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.TelegramService.Handler(c, s.logger)
+		h = s.TelegramService.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	case "victorops":
 		c := victorops.HandlerConfig{}
@@ -836,14 +955,25 @@ func (s *Service) createHandlerFromSpec(spec HandlerSpec) (handler, error) {
 		if err != nil {
 			return handler{}, err
 		}
-		h = s.VictorOpsService.Handler(c, s.logger)
+		h = s.VictorOpsService.Handler(c, ctx...)
 		h = newExternalHandler(h)
 	default:
 		err = fmt.Errorf("unsupported action kind %q", spec.Kind)
 	}
 	if spec.Match != "" {
 		// Wrap handler in match handler
-		h, err = newMatchHandler(spec.Match, h, s.logger)
+		handlerDiag := s.diag.WithHandlerContext(ctx...)
+		h, err = newMatchHandler(spec.Match, h, handlerDiag)
 	}
 	return handler{Spec: spec, Handler: h}, err
 }
+
+func (s *Service) IsInhibited(name string, tags models.Tags) bool {
+	return s.inhibitorLookup.IsInhibited(name, tags)
+}
+func (s *Service) AddInhibitor(in *alert.Inhibitor) {
+	s.inhibitorLookup.AddInhibitor(in)
+}
+func (s *Service) RemoveInhibitor(in *alert.Inhibitor) {
+	s.inhibitorLookup.RemoveInhibitor(in)
+}
