@@ -4,16 +4,23 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"log"
 	"sync"
 	"time"
 
+	"github.com/influxdata/kapacitor/edge"
+	"github.com/influxdata/kapacitor/keyvalue"
 	"github.com/influxdata/kapacitor/models"
 	"github.com/influxdata/kapacitor/udf/agent"
 )
 
 var ErrServerStopped = errors.New("server already stopped")
 
+type Diagnostic interface {
+	Error(msg string, err error, ctx ...keyvalue.T)
+
+	UDFLog(msg string)
+}
+
 // Server provides an implementation for the core communication with UDFs.
 // The Server provides only a partial implementation of udf.Interface as
 // it is expected that setup and teardown will be necessary to create a Server.
@@ -47,11 +54,9 @@ type Server struct {
 	// If abort fails after sometime this will be called
 	killCallback func()
 
-	pointIn chan models.Point
-	batchIn chan models.Batch
+	inMsg chan edge.Message
 
-	pointOut chan models.Point
-	batchOut chan models.Batch
+	outMsg chan edge.Message
 
 	stopped  bool
 	stopping chan struct{}
@@ -67,14 +72,17 @@ type Server struct {
 	keepalive        chan int64
 	keepaliveTimeout time.Duration
 
+	taskID string
+	nodeID string
+
 	in  agent.ByteReadReader
 	out io.WriteCloser
 
 	// Group for waiting on read/write goroutines
 	ioGroup sync.WaitGroup
 
-	mu     sync.Mutex
-	logger *log.Logger
+	mu   sync.Mutex
+	diag Diagnostic
 
 	responseBuf []byte
 
@@ -83,30 +91,33 @@ type Server struct {
 	snapshotResponse chan *agent.Response
 	restoreResponse  chan *agent.Response
 
-	batch *models.Batch
+	// Buffer up batch messages
+	begin  *agent.BeginBatch
+	points []edge.BatchPointMessage
 }
 
 func NewServer(
+	taskID, nodeID string,
 	in agent.ByteReadReader,
 	out io.WriteCloser,
-	l *log.Logger,
+	d Diagnostic,
 	timeout time.Duration,
 	abortCallback func(),
 	killCallback func(),
 ) *Server {
 	s := &Server{
+		taskID:           taskID,
+		nodeID:           nodeID,
 		in:               in,
 		out:              out,
-		logger:           l,
+		diag:             d,
 		requests:         make(chan *agent.Request),
 		keepalive:        make(chan int64, 1),
 		keepaliveTimeout: timeout,
 		abortCallback:    abortCallback,
 		killCallback:     killCallback,
-		pointIn:          make(chan models.Point),
-		batchIn:          make(chan models.Batch),
-		pointOut:         make(chan models.Point),
-		batchOut:         make(chan models.Batch),
+		inMsg:            make(chan edge.Message),
+		outMsg:           make(chan edge.Message),
 		infoResponse:     make(chan *agent.Response, 1),
 		initResponse:     make(chan *agent.Response, 1),
 		snapshotResponse: make(chan *agent.Response, 1),
@@ -116,17 +127,11 @@ func NewServer(
 	return s
 }
 
-func (s *Server) PointIn() chan<- models.Point {
-	return s.pointIn
+func (s *Server) In() chan<- edge.Message {
+	return s.inMsg
 }
-func (s *Server) BatchIn() chan<- models.Batch {
-	return s.batchIn
-}
-func (s *Server) PointOut() <-chan models.Point {
-	return s.pointOut
-}
-func (s *Server) BatchOut() <-chan models.Batch {
-	return s.batchOut
+func (s *Server) Out() <-chan edge.Message {
+	return s.outMsg
 }
 
 func (s *Server) setError(err error) {
@@ -223,8 +228,7 @@ func (s *Server) stop() error {
 
 	close(s.requests)
 
-	close(s.pointIn)
-	close(s.batchIn)
+	close(s.inMsg)
 
 	s.ioGroup.Wait()
 
@@ -266,6 +270,8 @@ func (s *Server) Init(options []*agent.Option) error {
 	req := &agent.Request{Message: &agent.Request_Init{
 		Init: &agent.InitRequest{
 			Options: options,
+			TaskID:  s.taskID,
+			NodeID:  s.nodeID,
 		},
 	}}
 	resp, err := s.doRequestResponse(req, s.initResponse)
@@ -297,7 +303,7 @@ func (s *Server) Snapshot() ([]byte, error) {
 // Request to restore a snapshot.
 func (s *Server) Restore(snapshot []byte) error {
 	req := &agent.Request{Message: &agent.Request_Restore{
-		Restore: &agent.RestoreRequest{snapshot},
+		Restore: &agent.RestoreRequest{Snapshot: snapshot},
 	}}
 	resp, err := s.doRequestResponse(req, s.restoreResponse)
 	if err != nil {
@@ -347,7 +353,7 @@ func (s *Server) doResponse(response *agent.Response, respC chan *agent.Response
 	select {
 	case respC <- response:
 	default:
-		s.logger.Printf("E! received %T without requesting it", response.Message)
+		s.diag.Error("received message without requesting it", fmt.Errorf("did not expect %T message", response.Message))
 	}
 }
 
@@ -398,7 +404,7 @@ func (s *Server) watchKeepalive() {
 				default:
 					// We failed to abort just kill it.
 					if s.killCallback != nil {
-						s.logger.Println("E! process not responding! killing")
+						s.diag.Error("killing process", errors.New("process not responding"))
 						s.killCallback()
 					}
 				}
@@ -425,7 +431,7 @@ func (s *Server) watchKeepalive() {
 				break
 			}
 			err = fmt.Errorf("keepalive timedout, last keepalive received was: %s", time.Unix(0, last))
-			s.logger.Println("E!", err)
+			s.diag.Error("encountered error", err)
 			return
 		case <-s.stopping:
 			return
@@ -436,25 +442,40 @@ func (s *Server) watchKeepalive() {
 // Write Requests
 func (s *Server) writeData() error {
 	defer s.out.Close()
+	var begin edge.BeginBatchMessage
 	for {
 		select {
-		case pt, ok := <-s.pointIn:
-			if ok {
-				err := s.writePoint(pt)
-				if err != nil {
-					return err
-				}
-			} else {
-				s.pointIn = nil
+		case m, ok := <-s.inMsg:
+			if !ok {
+				s.inMsg = nil
 			}
-		case bt, ok := <-s.batchIn:
-			if ok {
-				err := s.writeBatch(bt)
+			switch msg := m.(type) {
+			case edge.PointMessage:
+				err := s.writePoint(msg)
+				if err != nil {
+					return err
+				}
+			case edge.BeginBatchMessage:
+				begin = msg
+				err := s.writeBeginBatch(msg)
+				if err != nil {
+					return err
+				}
+			case edge.BatchPointMessage:
+				err := s.writeBatchPoint(begin.GroupID(), msg)
+				if err != nil {
+					return err
+				}
+			case edge.EndBatchMessage:
+				err := s.writeEndBatch(begin.Name(), begin.Time(), begin.GroupInfo(), msg)
+				if err != nil {
+					return err
+				}
+			case edge.BufferedBatchMessage:
+				err := s.writeBufferedBatch(msg)
 				if err != nil {
 					return err
 				}
-			} else {
-				s.batchIn = nil
 			}
 		case req, ok := <-s.requests:
 			if ok {
@@ -468,30 +489,31 @@ func (s *Server) writeData() error {
 		case <-s.aborting:
 			return s.err
 		}
-		if s.pointIn == nil && s.batchIn == nil && s.requests == nil {
+		if s.inMsg == nil && s.requests == nil {
 			break
 		}
 	}
 	return nil
 }
 
-func (s *Server) writePoint(pt models.Point) error {
-	strs, floats, ints := s.fieldsToTypedMaps(pt.Fields)
+func (s *Server) writePoint(p edge.PointMessage) error {
+	strs, floats, ints, bools := s.fieldsToTypedMaps(p.Fields())
 	udfPoint := &agent.Point{
-		Time:            pt.Time.UnixNano(),
-		Name:            pt.Name,
-		Database:        pt.Database,
-		RetentionPolicy: pt.RetentionPolicy,
-		Group:           string(pt.Group),
-		Dimensions:      pt.Dimensions.TagNames,
-		ByName:          pt.Dimensions.ByName,
-		Tags:            pt.Tags,
+		Time:            p.Time().UnixNano(),
+		Name:            p.Name(),
+		Database:        p.Database(),
+		RetentionPolicy: p.RetentionPolicy(),
+		Group:           string(p.GroupID()),
+		Dimensions:      p.Dimensions().TagNames,
+		ByName:          p.Dimensions().ByName,
+		Tags:            p.Tags(),
 		FieldsDouble:    floats,
 		FieldsInt:       ints,
 		FieldsString:    strs,
+		FieldsBool:      bools,
 	}
 	req := &agent.Request{
-		Message: &agent.Request_Point{udfPoint},
+		Message: &agent.Request_Point{Point: udfPoint},
 	}
 	return s.writeRequest(req)
 }
@@ -500,6 +522,7 @@ func (s *Server) fieldsToTypedMaps(fields models.Fields) (
 	strs map[string]string,
 	floats map[string]float64,
 	ints map[string]int64,
+	bools map[string]bool,
 ) {
 	for k, v := range fields {
 		switch value := v.(type) {
@@ -518,6 +541,11 @@ func (s *Server) fieldsToTypedMaps(fields models.Fields) (
 				ints = make(map[string]int64)
 			}
 			ints[k] = value
+		case bool:
+			if bools == nil {
+				bools = make(map[string]bool)
+			}
+			bools[k] = value
 		default:
 			panic("unsupported field value type")
 		}
@@ -529,6 +557,7 @@ func (s *Server) typeMapsToFields(
 	strs map[string]string,
 	floats map[string]float64,
 	ints map[string]int64,
+	bools map[string]bool,
 ) models.Fields {
 	fields := make(models.Fields)
 	for k, v := range strs {
@@ -540,53 +569,70 @@ func (s *Server) typeMapsToFields(
 	for k, v := range floats {
 		fields[k] = v
 	}
+	for k, v := range bools {
+		fields[k] = v
+	}
 	return fields
 }
 
-func (s *Server) writeBatch(b models.Batch) error {
+func (s *Server) writeBeginBatch(begin edge.BeginBatchMessage) error {
 	req := &agent.Request{
-		Message: &agent.Request_Begin{&agent.BeginBatch{
-			Name:   b.Name,
-			Group:  string(b.Group),
-			Tags:   b.Tags,
-			Size:   int64(len(b.Points)),
-			ByName: b.ByName,
-		}},
-	}
-	err := s.writeRequest(req)
-	if err != nil {
-		return err
-	}
-	rp := &agent.Request_Point{}
-	req.Message = rp
-	for _, pt := range b.Points {
-		strs, floats, ints := s.fieldsToTypedMaps(pt.Fields)
-		udfPoint := &agent.Point{
-			Time:         pt.Time.UnixNano(),
-			Group:        string(b.Group),
-			Tags:         pt.Tags,
-			FieldsDouble: floats,
-			FieldsInt:    ints,
-			FieldsString: strs,
-		}
-		rp.Point = udfPoint
-		err := s.writeRequest(req)
-		if err != nil {
-			return err
-		}
+		Message: &agent.Request_Begin{
+			Begin: &agent.BeginBatch{
+				Name:   begin.Name(),
+				Group:  string(begin.GroupID()),
+				Tags:   begin.Tags(),
+				Size:   int64(begin.SizeHint()),
+				ByName: begin.Dimensions().ByName,
+			}},
 	}
+	return s.writeRequest(req)
+}
 
-	req.Message = &agent.Request_End{
-		&agent.EndBatch{
-			Name:  b.Name,
-			Group: string(b.Group),
-			Tmax:  b.TMax.UnixNano(),
-			Tags:  b.Tags,
+func (s *Server) writeBatchPoint(group models.GroupID, bp edge.BatchPointMessage) error {
+	strs, floats, ints, bools := s.fieldsToTypedMaps(bp.Fields())
+	req := &agent.Request{
+		Message: &agent.Request_Point{
+			Point: &agent.Point{
+				Time:         bp.Time().UnixNano(),
+				Group:        string(group),
+				Tags:         bp.Tags(),
+				FieldsDouble: floats,
+				FieldsInt:    ints,
+				FieldsString: strs,
+				FieldsBool:   bools,
+			},
 		},
 	}
 	return s.writeRequest(req)
 }
 
+func (s *Server) writeEndBatch(name string, tmax time.Time, groupInfo edge.GroupInfo, end edge.EndBatchMessage) error {
+	req := &agent.Request{
+		Message: &agent.Request_End{
+			End: &agent.EndBatch{
+				Name:  name,
+				Group: string(groupInfo.ID),
+				Tmax:  tmax.UnixNano(),
+				Tags:  groupInfo.Tags,
+			},
+		},
+	}
+	return s.writeRequest(req)
+}
+
+func (s *Server) writeBufferedBatch(batch edge.BufferedBatchMessage) error {
+	if err := s.writeBeginBatch(batch.Begin()); err != nil {
+		return err
+	}
+	for _, bp := range batch.Points() {
+		if err := s.writeBatchPoint(batch.GroupID(), bp); err != nil {
+			return err
+		}
+	}
+	return s.writeEndBatch(batch.Name(), batch.Time(), batch.GroupInfo(), batch.End())
+}
+
 func (s *Server) writeRequest(req *agent.Request) error {
 	err := agent.WriteMessage(req, s.out)
 	if err != nil {
@@ -598,8 +644,7 @@ func (s *Server) writeRequest(req *agent.Request) error {
 // Read Responses from STDOUT of the process.
 func (s *Server) readData() error {
 	defer func() {
-		close(s.pointOut)
-		close(s.batchOut)
+		close(s.outMsg)
 	}()
 	for {
 		response, err := s.readResponse()
@@ -649,57 +694,65 @@ func (s *Server) handleResponse(response *agent.Response) error {
 	case *agent.Response_Restore:
 		s.doResponse(response, s.restoreResponse)
 	case *agent.Response_Error:
-		s.logger.Println("E!", msg.Error.Error)
+		s.diag.Error("received error message", errors.New(msg.Error.Error))
 		return errors.New(msg.Error.Error)
 	case *agent.Response_Begin:
-		s.batch = &models.Batch{
-			ByName: msg.Begin.ByName,
-			Points: make([]models.BatchPoint, 0, msg.Begin.Size),
-		}
+		s.begin = msg.Begin
+		s.points = make([]edge.BatchPointMessage, 0, msg.Begin.Size)
 	case *agent.Response_Point:
-		if s.batch != nil {
-			pt := models.BatchPoint{
-				Time: time.Unix(0, msg.Point.Time).UTC(),
-				Tags: msg.Point.Tags,
-				Fields: s.typeMapsToFields(
+		if s.points != nil {
+			bp := edge.NewBatchPointMessage(
+				s.typeMapsToFields(
 					msg.Point.FieldsString,
 					msg.Point.FieldsDouble,
 					msg.Point.FieldsInt,
+					msg.Point.FieldsBool,
 				),
-			}
-			s.batch.Points = append(s.batch.Points, pt)
+				msg.Point.Tags,
+				time.Unix(0, msg.Point.Time).UTC(),
+			)
+			s.points = append(s.points, bp)
 		} else {
-			pt := models.Point{
-				Time:            time.Unix(0, msg.Point.Time).UTC(),
-				Name:            msg.Point.Name,
-				Database:        msg.Point.Database,
-				RetentionPolicy: msg.Point.RetentionPolicy,
-				Group:           models.GroupID(msg.Point.Group),
-				Dimensions:      models.Dimensions{ByName: msg.Point.ByName, TagNames: msg.Point.Dimensions},
-				Tags:            msg.Point.Tags,
-				Fields: s.typeMapsToFields(
+			p := edge.NewPointMessage(
+				msg.Point.Name,
+				msg.Point.Database,
+				msg.Point.RetentionPolicy,
+				models.Dimensions{ByName: msg.Point.ByName, TagNames: msg.Point.Dimensions},
+				s.typeMapsToFields(
 					msg.Point.FieldsString,
 					msg.Point.FieldsDouble,
 					msg.Point.FieldsInt,
+					msg.Point.FieldsBool,
 				),
-			}
+				msg.Point.Tags,
+				time.Unix(0, msg.Point.Time).UTC(),
+			)
 			select {
-			case s.pointOut <- pt:
+			case s.outMsg <- p:
 			case <-s.aborting:
 				return s.err
 			}
 		}
 	case *agent.Response_End:
-		s.batch.Name = msg.End.Name
-		s.batch.TMax = time.Unix(0, msg.End.Tmax).UTC()
-		s.batch.Group = models.GroupID(msg.End.Group)
-		s.batch.Tags = msg.End.Tags
+		begin := edge.NewBeginBatchMessage(
+			msg.End.Name,
+			msg.End.Tags,
+			s.begin.ByName,
+			time.Unix(0, msg.End.Tmax).UTC(),
+			len(s.points),
+		)
+		bufferedBatch := edge.NewBufferedBatchMessage(
+			begin,
+			s.points,
+			edge.NewEndBatchMessage(),
+		)
 		select {
-		case s.batchOut <- *s.batch:
+		case s.outMsg <- bufferedBatch:
 		case <-s.aborting:
 			return s.err
 		}
-		s.batch = nil
+		s.begin = nil
+		s.points = nil
 	default:
 		panic(fmt.Sprintf("unexpected response message %T", msg))
 	}
